// file      : xsd-frontend/semantic-graph/schema.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX
#define XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/exception.hpp>

#include <xsd-frontend/semantic-graph/elements.hxx>
#include <xsd-frontend/semantic-graph/namespace.hxx>

#include <iosfwd>

namespace XSDFrontend
{
  namespace SemanticGraph
  {
    //
    //
    typedef
    boost::filesystem::filesystem_error
    InvalidPath;

    typedef
    boost::filesystem::path
    Path;

    std::wostream&
    operator<< (std::wostream& os, Path const& path);


    //
    //
    class Schema;


    //
    //
    class Implies: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Implies (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Sources: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Sources (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Includes: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Includes (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Imports: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Imports (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Schema: public virtual Container,
                  public virtual Scope,
                  private Bits::Graph<Node, Edge>,
                  public NonCopyable
    {
    public:
      Schema (UnsignedLong line, UnsignedLong column)
          : SemanticGraph::Node (line, column), contained_ (0)
      {
      }

    public:
      Boolean
      contained_p () const
      {
        return contained_ != 0;
      }

      Contains&
      contained () const
      {
        return *contained_;
      }

      virtual NamesIteratorPair
      find (Name const& name) const;

    private:
      Bits::Graph<SemanticGraph::Node, Edge>&
      graph ()
      {
        return *this;
      }

    public:
      using Bits::Graph<SemanticGraph::Node, Edge>::new_edge;

      template <typename T>
      T&
      new_node (UnsignedLong line, UnsignedLong column)
      {
        return graph ().new_node<T> (line, column);
      }

      template <typename T, typename A0>
      T&
      new_node (UnsignedLong line, UnsignedLong column,
                A0 const& a0)
      {
        return graph ().new_node<T> (line, column, a0);
      }

      template <typename T, typename A0, typename A1>
      T&
      new_node (UnsignedLong line, UnsignedLong column,
                A0 const& a0, A1 const& a1)
      {
        return graph ().new_node<T> (line, column, a0, a1);
      }

      template <typename T, typename A0, typename A1, typename A2>
      T&
      new_node (UnsignedLong line, UnsignedLong column,
                A0 const& a0, A1 const& a1, A2 const& a2)
      {
        return graph ().new_node<T> (line, column, a0, a1, a2);
      }

    protected:
      //@@ gcc bug #21146
      //
      friend class Bits::Graph<SemanticGraph::Node, Edge>;

      using Scope::add_edge_left;
      using Container::add_edge_left;

      using Node::add_edge_right;

      Void
      add_edge_right (Contains& c)
      {
        contained_ = &c;
      }

    private:
      Contains* contained_;

      mutable NamesList names_;
    };
  }
}

#endif  // XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX
