// file      : xsd-frontend/xml.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_XML_HXX
#define XSD_FRONTEND_XML_HXX

#include <xsd-frontend/types.hxx>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/impl/XSDElementNSImpl.hpp>

#include <xercesc/util/XMLString.hpp>

//@@ tmp
//#include <iostream>

namespace XSDFrontend
{
  namespace XML
  {
    namespace Xerces = xercesc;

    inline
    String
    transcode (XMLCh const* s)
    {
      String r (Xerces::XMLString::stringLen (s), L'0');

      for (Size i (0); *s != XMLCh (0); ++s, ++i)
      {
        r[i] = *s;
      }

      return r;
    }


    inline
    XMLCh*
    transcode (String const& s)
    {
      Size l (s.length ());

      XMLCh* r (new XMLCh[l + 1]);
      XMLCh* ir (r);

      for (Size i (0); i < l; ++ir, ++i)
      {
        *ir = static_cast<XMLCh>(s[i]);
        //std::wcerr << s[i] << "->" << *ir << std::endl;
      }

      *ir = XMLCh (0);

      // std::wcerr << r << std::endl;

      return r;
    }

    class Element
    {
    public:
      Element (Xerces::DOMElement* e)
          : e_ (e),
            name_ (transcode (e->getLocalName ())),
            namespace__ (transcode (e->getNamespaceURI ()))
      {
      }

      String
      name () const
      {
        return name_;
      }

      String
      namespace_ () const
      {
        return namespace__;
      }

    public:
      UnsignedLong
      line () const
      {
        using Xerces::XSDElementNSImpl;

        //@@ cache

        if (XSDElementNSImpl* e = dynamic_cast<XSDElementNSImpl*> (e_))
          return e->getLineNo ();
        else
          return 0;
      }

      UnsignedLong
      column () const
      {
        using Xerces::XSDElementNSImpl;

        //@@ cache

        if (XSDElementNSImpl* e = dynamic_cast<XSDElementNSImpl*> (e_))
          return e->getColumnNo ();
        else
          return 0;
      }

    public:
      Element
      parent () const
      {
        return dynamic_cast<Xerces::DOMElement*>(e_->getParentNode ());
      }

    public:
      String
      operator[] (String const& s) const
      {
        XMLCh* name (transcode (s));
        XMLCh const* value (e_->getAttribute (name));
        delete[] name;

        return transcode (value);
      }

    public:
      Xerces::DOMElement*
      dom_element () const
      {
        return e_;
      }

    private:

    private:
      Xerces::DOMElement* e_;

      String name_;
      String namespace__;
    };

    inline
    String
    prefix (String const& n)
    {
      Size i (0);
      while (i < n.length () && n[i] != L':') ++i;

      //std::wcerr << "prefix " << n << " "
      //           << String (n, i == n.length () ? i : 0, i) << std::endl;

      return String (n, i == n.length () ? i : 0, i);
    }

    inline
    String
    uq_name (String const& n)
    {
      Size i (0);
      while (i < n.length () && n[i] != L':') ++i;

      return String (n.c_str () + (i == n.length () ? 0 : i + 1));
    }

    inline
    String
    ns_name (Element const& e, String const& n)
    {
      XMLCh* p (0);

      String wp (prefix (n));

      if (!wp.empty ())
        p = transcode (wp);

      XMLCh const* xns (e.dom_element ()->lookupNamespaceURI (p));
      String ns (xns ? transcode (xns) : String (L""));
      delete[] p;
      return ns;
    }

    //@@ Can get rid of use_default crap. See RT.
    //
    inline
    String
    ns_prefix (Element const& e,
               String const& ns,
               Boolean use_default)
    {
      XMLCh* p (transcode (ns));
      XMLCh const* xns (e.dom_element ()->lookupNamespacePrefix (p, use_default));
      String prefix (xns ? transcode (xns) : String (L""));
      delete[] p;
      return prefix;
    }

    inline
    String
    fq_name (Element const& e, String const& n)
    {
      String ns (ns_name (e, n));
      String un (uq_name (n));

      return ns.empty () ? un : (ns + L'#' + un);
    }
  }
}

#endif  // XSD_FRONTEND_XML_HXX
